@@ -2,7 +2,7 @@
 #
 # Automated Testing Framework (atf)
 #
-# Copyright (c) 2007 The NetBSD Foundation, Inc.
+# Copyright (c) 2007, 2008 The NetBSD Foundation, Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -13,13 +13,6 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. All advertising materials mentioning features or use of this
-#    software must display the following acknowledgement:
-#        This product includes software developed by the NetBSD
-#        Foundation, Inc. and its contributors.
-# 4. Neither the name of The NetBSD Foundation nor the names of its
-#    contributors may be used to endorse or promote products derived
-#    from this software without specific prior written permission.
 #
 # THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND
 # CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
@@ -67,7 +60,7 @@ guess_topdir() {
     olddir=$(pwd)
     topdir=$(pwd)
     while [ ${topdir} != / ]; do
-        if [ -f ./atf.hpp ]; then
+        if [ -f ./atf-c.h ]; then
             break
         else
             cd ..
@@ -96,7 +89,9 @@ find_sources() {
               -name "*.am" -o \
               -name "*.at" -o \
               -name "*.awk" -o \
+              -name "*.c" -o \
               -name "*.cpp" -o \
+              -name "*.h" -o \
               -name "*.hpp" -o \
               -name "*.m4" -o \
               -name "*.sh" \
@@ -104,6 +99,7 @@ find_sources() {
               \! -path "*autom4te*" -a \
               -type f -a \
               \! -name "aclocal.m4" \
+              \! -name "bconfig.h" \
               \! -name "*.so.*" \
            \)
 }
@@ -121,6 +117,9 @@ guess_formats() {
         *.[0-9])
             echo common man
             ;;
+        *.c|*.h)
+            echo common c
+            ;;
         *.cpp|*.hpp)
             echo common cpp
             ;;
@@ -154,7 +153,7 @@ check_file() {
 #
 main() {
     topdir=$(guess_topdir)
-    if [ ! -f ${topdir}/atf.hpp ]; then
+    if [ ! -f ${topdir}/atf-c.h ]; then
         err "Could not locate the project's top directory"
     fi
 
