@@ -46,54 +46,13 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "atf-c/error.h"
 #include "atf-c/fs.h"
+#include "atf-c/process.h"
 #include "atf-c/sanity.h"
 #include "atf-c/text.h"
 #include "atf-c/user.h"
 
-/* ---------------------------------------------------------------------
- * The "child" error type.
- * --------------------------------------------------------------------- */
-
-/* XXX The need for this conditional here is extremely ugly.  This
- * exception should belong to another module with more process-related
- * utilities. */
-#if !defined(HAVE_UNMOUNT)
-struct child_error_data {
-    char m_cmd[4096];
-    int m_state;
-};
-typedef struct child_error_data child_error_data_t;
-
-static
-void
-child_format(const atf_error_t err, char *buf, size_t buflen)
-{
-    const child_error_data_t *data;
-
-    PRE(atf_error_is(err, "child"));
-
-    data = atf_error_data(err);
-    snprintf(buf, buflen, "Unknown error while executing \"%s\"; "
-             "exit state was %d", data->m_cmd, data->m_state);
-}
-
-static
-atf_error_t
-child_error(const char *cmd, int state)
-{
-    atf_error_t err;
-    child_error_data_t data;
-
-    snprintf(data.m_cmd, sizeof(data.m_cmd), "%s", cmd);
-    data.m_state = state;
-
-    err = atf_error_new("child", &data, sizeof(data), child_format);
-
-    return err;
-}
-#endif /* !defined(HAVE_UNMOUNT) */
-
 /* ---------------------------------------------------------------------
  * The "unknown_file_type" error type.
  * --------------------------------------------------------------------- */
@@ -228,6 +187,61 @@ out:
     return err;
 }
 
+static
+atf_error_t
+copy_contents(const atf_fs_path_t *p, char **buf)
+{
+    atf_error_t err;
+    char *str;
+
+    str = (char *)malloc(atf_dynstr_length(&p->m_data) + 1);
+    if (str == NULL)
+        err = atf_no_memory_error();
+    else {
+        strcpy(str, atf_dynstr_cstring(&p->m_data));
+        *buf = str;
+        err = atf_no_error();
+    }
+
+    return err;
+}
+
+static
+atf_error_t
+do_mkdtemp(char *tmpl)
+{
+    atf_error_t err;
+
+    PRE(strstr(tmpl, "XXXXXX") != NULL);
+
+    if (mkdtemp(tmpl) == NULL)
+        err = atf_libc_error(errno, "Cannot create temporary directory "
+                             "with template '%s'", tmpl);
+    else
+        err = atf_no_error();
+
+    return err;
+}
+
+static
+atf_error_t
+do_mkstemp(char *tmpl, int *fdout)
+{
+    atf_error_t err;
+
+    PRE(strstr(tmpl, "XXXXXX") != NULL);
+
+    *fdout = mkstemp(tmpl);
+    if (*fdout == -1)
+        err = atf_libc_error(errno, "Cannot create temporary file "
+                             "with template '%s'", tmpl);
+
+    else
+        err = atf_no_error();
+
+    return err;
+}
+
 static
 atf_error_t
 do_unmount(const atf_fs_path_t *p)
@@ -264,11 +278,7 @@ do_unmount(const atf_fs_path_t *p)
 
         err = atf_text_format(&cmd, "umount '%s'", pastr);
         if (!atf_is_error(err)) {
-            int state = system(cmd);
-            if (state == -1)
-                err = atf_libc_error(errno, "Failed to run \"%s\"", cmd);
-            else if (!WIFEXITED(state) || WEXITSTATUS(state) != EXIT_SUCCESS)
-                err = child_error(cmd, state);
+            err = atf_process_system(cmd);
             free(cmd);
         }
     }
@@ -337,6 +347,20 @@ out:
     return err;
 }
 
+static
+void
+replace_contents(atf_fs_path_t *p, const char *buf)
+{
+    atf_error_t err;
+
+    PRE(atf_dynstr_length(&p->m_data) == strlen(buf));
+
+    atf_dynstr_clear(&p->m_data);
+    err = atf_dynstr_append_fmt(&p->m_data, "%s", buf);
+
+    INV(!atf_is_error(err));
+}
+
 /* ---------------------------------------------------------------------
  * The "atf_fs_path" type.
  * --------------------------------------------------------------------- */
@@ -616,6 +640,12 @@ atf_fs_stat_get_inode(const atf_fs_stat_t *st)
     return st->m_sb.st_ino;
 }
 
+off_t
+atf_fs_stat_get_size(const atf_fs_stat_t *st)
+{
+    return st->m_sb.st_size;
+}
+
 int
 atf_fs_stat_get_type(const atf_fs_stat_t *st)
 {
@@ -813,14 +843,60 @@ atf_error_t
 atf_fs_mkdtemp(atf_fs_path_t *p)
 {
     atf_error_t err;
-    char *tmpl;
+    char *buf;
 
-    tmpl = p->m_data.m_data; /* XXX: Ugly */
-    PRE(strstr(tmpl, "XXXXXX") != NULL);
+    err = copy_contents(p, &buf);
+    if (atf_is_error(err))
+        goto out;
 
-    if (mkdtemp(tmpl) == NULL)
-        err = atf_libc_error(errno, "Cannot create temporary directory "
-                             "with template '%s'", tmpl);
+    err = do_mkdtemp(buf);
+    if (atf_is_error(err))
+        goto out_buf;
+
+    replace_contents(p, buf);
+
+    INV(!atf_is_error(err));
+out_buf:
+    free(buf);
+out:
+    return err;
+}
+
+atf_error_t
+atf_fs_mkstemp(atf_fs_path_t *p, int *fdout)
+{
+    atf_error_t err;
+    char *buf;
+    int fd;
+
+    err = copy_contents(p, &buf);
+    if (atf_is_error(err))
+        goto out;
+
+    err = do_mkstemp(buf, &fd);
+    if (atf_is_error(err))
+        goto out_buf;
+
+    replace_contents(p, buf);
+    *fdout = fd;
+
+    INV(!atf_is_error(err));
+out_buf:
+    free(buf);
+out:
+    return err;
+}
+
+atf_error_t
+atf_fs_unlink(const atf_fs_path_t *p)
+{
+    atf_error_t err;
+    const char *path;
+
+    path = atf_fs_path_cstring(p);
+
+    if (unlink(path) != 0)
+        err = atf_libc_error(errno, "Cannot unlink file: '%s'", path);
     else
         err = atf_no_error();
 
