@@ -34,6 +34,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -43,8 +44,9 @@
 
 #include "atf-c/config.h"
 #include "atf-c/env.h"
+#include "atf-c/error.h"
 #include "atf-c/fs.h"
-#include "atf-c/io.h"
+#include "atf-c/process.h"
 #include "atf-c/sanity.h"
 #include "atf-c/signals.h"
 #include "atf-c/tc.h"
@@ -65,7 +67,6 @@ static atf_error_t fork_body(const atf_tc_t *, const atf_fs_path_t *,
                              atf_tcr_t *);
 static atf_error_t fork_cleanup(const atf_tc_t *, const atf_fs_path_t *);
 static atf_error_t get_tc_result(const atf_fs_path_t *, atf_tcr_t *);
-static atf_error_t parse_tc_result(int, atf_tcr_t *);
 static atf_error_t program_timeout(pid_t, const atf_tc_t *,
                                    struct timeout_data *);
 static void unprogram_timeout(struct timeout_data *);
@@ -82,12 +83,15 @@ static atf_error_t check_prog_in_dir(const char *, void *);
 static atf_error_t check_requirements(const atf_tc_t *);
 static void cleanup_child(const atf_tc_t *, const atf_fs_path_t *)
             ATF_DEFS_ATTRIBUTE_NORETURN;
+static void fail_internal(const char *, int, const char *, const char *,
+                          const char *, va_list,
+                          void (*)(const char *, ...));
 static void fatal_atf_error(const char *, atf_error_t)
             ATF_DEFS_ATTRIBUTE_NORETURN;
 static void fatal_libc_error(const char *, int)
             ATF_DEFS_ATTRIBUTE_NORETURN;
 static atf_error_t prepare_child(const atf_tc_t *, const atf_fs_path_t *);
-static void write_tcr(const atf_tc_t *, const atf_tcr_t *);
+static void write_tcr(const atf_tcr_t *);
 
 /* ---------------------------------------------------------------------
  * The "atf_tc" type.
@@ -443,11 +447,11 @@ fork_body(const atf_tc_t *tc, const atf_fs_path_t *workdir, atf_tcr_t *tcr)
     atf_error_t err;
     pid_t pid;
 
-    pid = fork();
-    if (pid == -1) {
-        err = atf_libc_error(errno, "Cannot fork to run test case body "
-                             "of %s", tc->m_ident);
-    } else if (pid == 0) {
+    err = atf_process_fork(&pid);
+    if (atf_is_error(err))
+        goto out;
+
+    if (pid == 0) {
         body_child(tc, workdir);
         UNREACHABLE;
         abort();
@@ -455,6 +459,7 @@ fork_body(const atf_tc_t *tc, const atf_fs_path_t *workdir, atf_tcr_t *tcr)
         err = body_parent(tc, workdir, pid, tcr);
     }
 
+out:
     return err;
 }
 
@@ -468,11 +473,11 @@ fork_cleanup(const atf_tc_t *tc, const atf_fs_path_t *workdir)
     if (tc->m_cleanup == NULL)
         err = atf_no_error();
     else {
-        pid = fork();
-        if (pid == -1) {
-            err = atf_libc_error(errno, "Cannot fork to run test case "
-                                 "cleanup of %s", tc->m_ident);
-        } else if (pid == 0) {
+        err = atf_process_fork(&pid);
+        if (atf_is_error(err))
+            goto out;
+
+        if (pid == 0) {
             cleanup_child(tc, workdir);
             UNREACHABLE;
             abort();
@@ -481,6 +486,7 @@ fork_cleanup(const atf_tc_t *tc, const atf_fs_path_t *workdir)
         }
     }
 
+out:
     return err;
 }
 
@@ -506,7 +512,7 @@ get_tc_result(const atf_fs_path_t *workdir, atf_tcr_t *tcr)
         goto out_tcrfile;
     }
 
-    err = parse_tc_result(fd, tcr);
+    err = atf_tcr_deserialize(tcr, fd);
 
     close(fd);
 out_tcrfile:
@@ -515,56 +521,13 @@ out:
     return err;
 }
 
-static
-atf_error_t
-parse_tc_result(int fd, atf_tcr_t *tcr)
-{
-    atf_error_t err;
-    atf_dynstr_t state, reason;
-
-    err = atf_dynstr_init(&state);
-    if (atf_is_error(err))
-        goto out;
-
-    err = atf_dynstr_init(&reason);
-    if (atf_is_error(err))
-        goto out_state;
-
-    err = atf_io_readline(fd, &state);
-    if (atf_is_error(err))
-        goto out_reason;
-
-    if (atf_equal_dynstr_cstring(&state, "passed"))
-        err = atf_tcr_init(tcr, atf_tcr_passed_state);
-    else {
-        err = atf_io_readline(fd, &reason);
-        if (atf_is_error(err))
-            goto out_reason;
-
-        if (atf_equal_dynstr_cstring(&state, "failed"))
-            err = atf_tcr_init_reason_fmt(tcr, atf_tcr_failed_state, "%s",
-                                          atf_dynstr_cstring(&reason));
-        else if (atf_equal_dynstr_cstring(&state, "skipped"))
-            err = atf_tcr_init_reason_fmt(tcr, atf_tcr_skipped_state, "%s",
-                                          atf_dynstr_cstring(&reason));
-        else
-            UNREACHABLE;
-    }
-
-out_reason:
-    atf_dynstr_fini(&reason);
-out_state:
-    atf_dynstr_fini(&state);
-out:
-    return err;
-}
-
 /*
  * Child-only stuff.
  */
 
 static const atf_tc_t *current_tc = NULL;
 static const atf_fs_path_t *current_workdir = NULL;
+static size_t current_tc_fail_count = 0;
 
 static
 atf_error_t
@@ -575,6 +538,7 @@ prepare_child(const atf_tc_t *tc, const atf_fs_path_t *workdir)
 
     current_tc = tc;
     current_workdir = workdir;
+    current_tc_fail_count = 0;
 
     ret = setpgid(getpid(), 0);
     INV(ret != -1);
@@ -651,7 +615,12 @@ body_child(const atf_tc_t *tc, const atf_fs_path_t *workdir)
     if (atf_is_error(err))
         goto print_err;
     tc->m_body(tc);
-    atf_tc_pass();
+
+    if (current_tc_fail_count == 0)
+        atf_tc_pass();
+    else
+        atf_tc_fail("%d checks failed; see output for more details",
+                    current_tc_fail_count);
 
     UNREACHABLE;
     abort();
@@ -915,7 +884,7 @@ fatal_libc_error(const char *prefix, int err)
 
 static
 void
-write_tcr(const atf_tc_t *tc, const atf_tcr_t *tcr)
+write_tcr(const atf_tcr_t *tcr)
 {
     atf_error_t err;
     int fd;
@@ -934,34 +903,9 @@ write_tcr(const atf_tc_t *tc, const atf_tcr_t *tcr)
     if (fd == -1)
         fatal_libc_error("Cannot write test case results", errno);
 
-    {
-        const atf_tcr_state_t s = atf_tcr_get_state(tcr);
-        const char *str;
-
-        if (s == atf_tcr_passed_state)
-            str = "passed";
-        else if (s == atf_tcr_failed_state)
-            str = "failed";
-        else if (s == atf_tcr_skipped_state)
-            str = "skipped";
-        else {
-            UNREACHABLE;
-            str = NULL;
-        }
-
-        err = atf_io_write_fmt(fd, "%s\n", str);
-        if (atf_is_error(err))
-            fatal_atf_error("Cannot write test case results", err);
-    }
-
-    {
-        if (atf_tcr_has_reason(tcr)) {
-            const atf_dynstr_t *r = atf_tcr_get_reason(tcr);
-            err = atf_io_write_fmt(fd, "%s\n", atf_dynstr_cstring(r));
-            if (atf_is_error(err))
-                fatal_atf_error("Cannot write test case results", err);
-        }
-    }
+    err = atf_tcr_serialize(tcr, fd);
+    if (atf_is_error(err))
+        fatal_atf_error("Cannot write test case results", err);
 
     close(fd);
 }
@@ -981,13 +925,85 @@ atf_tc_fail(const char *fmt, ...)
     if (atf_is_error(err))
         abort();
 
-    write_tcr(current_tc, &tcr);
+    write_tcr(&tcr);
 
     atf_tcr_fini(&tcr);
 
     exit(EXIT_SUCCESS);
 }
 
+void
+atf_tc_fail_nonfatal(const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    vfprintf(stderr, fmt, ap);
+    va_end(ap);
+    fprintf(stderr, "\n");
+
+    current_tc_fail_count++;
+}
+
+void
+atf_tc_fail_check(const char *file, int line, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    fail_internal(file, line, "Check failed", "*** ", fmt, ap,
+                  atf_tc_fail_nonfatal);
+    va_end(ap);
+}
+
+void
+atf_tc_fail_requirement(const char *file, int line, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    fail_internal(file, line, "Requirement failed", "", fmt, ap,
+                  atf_tc_fail);
+    va_end(ap);
+
+    UNREACHABLE;
+    abort();
+}
+
+static
+void
+fail_internal(const char *file, int line, const char *reason,
+              const char *prefix, const char *fmt, va_list ap,
+              void (*failfunc)(const char *, ...))
+{
+    va_list ap2;
+    atf_error_t err;
+    atf_dynstr_t msg;
+
+    err = atf_dynstr_init_fmt(&msg, "%s%s:%d: %s: ", prefix, file, line,
+                              reason);
+    if (atf_is_error(err))
+        goto backup;
+
+    va_copy(ap2, ap);
+    err = atf_dynstr_append_ap(&msg, fmt, ap2);
+    va_end(ap2);
+    if (atf_is_error(err)) {
+        atf_dynstr_fini(&msg);
+        goto backup;
+    }
+
+    va_copy(ap2, ap);
+    failfunc("%s", atf_dynstr_cstring(&msg));
+    atf_dynstr_fini(&msg);
+    return;
+
+backup:
+    va_copy(ap2, ap);
+    failfunc(fmt, ap2);
+    va_end(ap2);
+}
+
 void
 atf_tc_pass(void)
 {
@@ -1000,7 +1016,7 @@ atf_tc_pass(void)
     if (atf_is_error(err))
         abort();
 
-    write_tcr(current_tc, &tcr);
+    write_tcr(&tcr);
 
     atf_tcr_fini(&tcr);
 
@@ -1032,7 +1048,7 @@ atf_tc_skip(const char *fmt, ...)
     if (atf_is_error(err))
         abort();
 
-    write_tcr(current_tc, &tcr);
+    write_tcr(&tcr);
 
     atf_tcr_fini(&tcr);
 
