@@ -31,7 +31,7 @@
 #define ATF_C_TC_H
 
 #include <atf-c/defs.h>
-#include <atf-c/error.h>
+#include <atf-c/error_fwd.h>
 #include <atf-c/map.h>
 #include <atf-c/object.h>
 
@@ -107,8 +107,14 @@ atf_error_t atf_tc_run(const atf_tc_t *, struct atf_tcr *,
 
 /* To be run from test case bodies only. */
 void atf_tc_fail(const char *, ...) ATF_DEFS_ATTRIBUTE_NORETURN;
+void atf_tc_fail_nonfatal(const char *, ...);
 void atf_tc_pass(void) ATF_DEFS_ATTRIBUTE_NORETURN;
 void atf_tc_require_prog(const char *);
 void atf_tc_skip(const char *, ...) ATF_DEFS_ATTRIBUTE_NORETURN;
 
+/* To be run from test case bodies only; internal to macros.h. */
+void atf_tc_fail_check(const char *, int, const char *, ...);
+void atf_tc_fail_requirement(const char *, int, const char *, ...)
+     ATF_DEFS_ATTRIBUTE_NORETURN;
+
 #endif /* ATF_C_TC_H */
