@@ -1,7 +1,7 @@
 /*
  * Automated Testing Framework (atf)
  *
- * Copyright (c) 2007, 2008 The NetBSD Foundation, Inc.
+ * Copyright (c) 2007, 2008, 2009 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,12 +42,12 @@
 #include "atf-c/fs.h"
 #include "atf-c/user.h"
 
+#include "h_macros.h"
+
 /* ---------------------------------------------------------------------
  * Auxiliary functions.
  * --------------------------------------------------------------------- */
 
-#define CE(stm) ATF_CHECK(!atf_is_error(stm))
-
 static
 void
 create_dir(const char *p, int mode)
@@ -129,9 +129,9 @@ ATF_TC_BODY(path_normalize, tc)
         printf("Input          : >%s<\n", t->in);
         printf("Expected output: >%s<\n", t->out);
 
-        CE(atf_fs_path_init_fmt(&p, "%s", t->in));
+        RE(atf_fs_path_init_fmt(&p, "%s", t->in));
         printf("Output         : >%s<\n", atf_fs_path_cstring(&p));
-        ATF_CHECK(strcmp(atf_fs_path_cstring(&p), t->out) == 0);
+        ATF_REQUIRE(strcmp(atf_fs_path_cstring(&p), t->out) == 0);
         atf_fs_path_fini(&p);
 
         printf("\n");
@@ -147,14 +147,14 @@ ATF_TC_BODY(path_copy, tc)
 {
     atf_fs_path_t str, str2;
 
-    CE(atf_fs_path_init_fmt(&str, "foo"));
-    CE(atf_fs_path_copy(&str2, &str));
+    RE(atf_fs_path_init_fmt(&str, "foo"));
+    RE(atf_fs_path_copy(&str2, &str));
 
-    ATF_CHECK(atf_equal_fs_path_fs_path(&str, &str2));
+    ATF_REQUIRE(atf_equal_fs_path_fs_path(&str, &str2));
 
-    CE(atf_fs_path_append_fmt(&str2, "bar"));
+    RE(atf_fs_path_append_fmt(&str2, "bar"));
 
-    ATF_CHECK(!atf_equal_fs_path_fs_path(&str, &str2));
+    ATF_REQUIRE(!atf_equal_fs_path_fs_path(&str, &str2));
 
     atf_fs_path_fini(&str2);
     atf_fs_path_fini(&str);
@@ -187,13 +187,13 @@ ATF_TC_BODY(path_is_absolute, tc)
         printf("Input          : %s\n", t->in);
         printf("Expected result: %s\n", t->abs ? "true" : "false");
 
-        CE(atf_fs_path_init_fmt(&p, "%s", t->in));
+        RE(atf_fs_path_init_fmt(&p, "%s", t->in));
         printf("Result         : %s\n",
                atf_fs_path_is_absolute(&p) ? "true" : "false");
         if (t->abs)
-            ATF_CHECK(atf_fs_path_is_absolute(&p));
+            ATF_REQUIRE(atf_fs_path_is_absolute(&p));
         else
-            ATF_CHECK(!atf_fs_path_is_absolute(&p));
+            ATF_REQUIRE(!atf_fs_path_is_absolute(&p));
         atf_fs_path_fini(&p);
 
         printf("\n");
@@ -227,13 +227,13 @@ ATF_TC_BODY(path_is_root, tc)
         printf("Input          : %s\n", t->in);
         printf("Expected result: %s\n", t->root ? "true" : "false");
 
-        CE(atf_fs_path_init_fmt(&p, "%s", t->in));
+        RE(atf_fs_path_init_fmt(&p, "%s", t->in));
         printf("Result         : %s\n",
                atf_fs_path_is_root(&p) ? "true" : "false");
         if (t->root)
-            ATF_CHECK(atf_fs_path_is_root(&p));
+            ATF_REQUIRE(atf_fs_path_is_root(&p));
         else
-            ATF_CHECK(!atf_fs_path_is_root(&p));
+            ATF_REQUIRE(!atf_fs_path_is_root(&p));
         atf_fs_path_fini(&p);
 
         printf("\n");
@@ -267,10 +267,10 @@ ATF_TC_BODY(path_branch_path, tc)
         printf("Input          : %s\n", t->in);
         printf("Expected output: %s\n", t->branch);
 
-        CE(atf_fs_path_init_fmt(&p, "%s", t->in));
-        CE(atf_fs_path_branch_path(&p, &bp));
+        RE(atf_fs_path_init_fmt(&p, "%s", t->in));
+        RE(atf_fs_path_branch_path(&p, &bp));
         printf("Output         : %s\n", atf_fs_path_cstring(&bp));
-        ATF_CHECK(strcmp(atf_fs_path_cstring(&bp), t->branch) == 0);
+        ATF_REQUIRE(strcmp(atf_fs_path_cstring(&bp), t->branch) == 0);
         atf_fs_path_fini(&p);
 
         printf("\n");
@@ -305,10 +305,10 @@ ATF_TC_BODY(path_leaf_name, tc)
         printf("Input          : %s\n", t->in);
         printf("Expected output: %s\n", t->leaf);
 
-        CE(atf_fs_path_init_fmt(&p, "%s", t->in));
-        CE(atf_fs_path_leaf_name(&p, &ln));
+        RE(atf_fs_path_init_fmt(&p, "%s", t->in));
+        RE(atf_fs_path_leaf_name(&p, &ln));
         printf("Output         : %s\n", atf_dynstr_cstring(&ln));
-        ATF_CHECK(atf_equal_dynstr_cstring(&ln, t->leaf));
+        ATF_REQUIRE(atf_equal_dynstr_cstring(&ln, t->leaf));
         atf_fs_path_fini(&p);
 
         printf("\n");
@@ -344,12 +344,12 @@ ATF_TC_BODY(path_append, tc)
         printf("Append         : >%s<\n", t->ap);
         printf("Expected output: >%s<\n", t->out);
 
-        CE(atf_fs_path_init_fmt(&p, "%s", t->in));
+        RE(atf_fs_path_init_fmt(&p, "%s", t->in));
 
-        CE(atf_fs_path_append_fmt(&p, "%s", t->ap));
+        RE(atf_fs_path_append_fmt(&p, "%s", t->ap));
 
         printf("Output         : >%s<\n", atf_fs_path_cstring(&p));
-        ATF_CHECK(strcmp(atf_fs_path_cstring(&p), t->out) == 0);
+        ATF_REQUIRE(strcmp(atf_fs_path_cstring(&p), t->out) == 0);
 
         atf_fs_path_fini(&p);
 
@@ -368,25 +368,25 @@ ATF_TC_BODY(path_to_absolute, tc)
     const char *names[] = { ".", "dir", NULL };
     const char **n;
 
-    ATF_CHECK(mkdir("dir", 0755) != -1);
+    ATF_REQUIRE(mkdir("dir", 0755) != -1);
 
     for (n = names; *n != NULL; n++) {
         atf_fs_path_t p, p2;
         atf_fs_stat_t st1, st2;
 
-        CE(atf_fs_path_init_fmt(&p, "%s", *n));
-        CE(atf_fs_stat_init(&st1, &p));
+        RE(atf_fs_path_init_fmt(&p, "%s", *n));
+        RE(atf_fs_stat_init(&st1, &p));
         printf("Relative path: %s\n", atf_fs_path_cstring(&p));
 
-        CE(atf_fs_path_to_absolute(&p, &p2));
+        RE(atf_fs_path_to_absolute(&p, &p2));
         printf("Absolute path: %s\n", atf_fs_path_cstring(&p2));
 
-        ATF_CHECK(atf_fs_path_is_absolute(&p2));
-        CE(atf_fs_stat_init(&st2, &p2));
+        ATF_REQUIRE(atf_fs_path_is_absolute(&p2));
+        RE(atf_fs_stat_init(&st2, &p2));
 
-        ATF_CHECK_EQUAL(atf_fs_stat_get_device(&st1),
+        ATF_REQUIRE_EQ(atf_fs_stat_get_device(&st1),
                         atf_fs_stat_get_device(&st2));
-        ATF_CHECK_EQUAL(atf_fs_stat_get_inode(&st1),
+        ATF_REQUIRE_EQ(atf_fs_stat_get_inode(&st1),
                         atf_fs_stat_get_inode(&st2));
 
         atf_fs_stat_fini(&st2);
@@ -407,14 +407,14 @@ ATF_TC_BODY(path_equal, tc)
 {
     atf_fs_path_t p1, p2;
 
-    CE(atf_fs_path_init_fmt(&p1, "foo"));
+    RE(atf_fs_path_init_fmt(&p1, "foo"));
 
-    CE(atf_fs_path_init_fmt(&p2, "foo"));
-    ATF_CHECK(atf_equal_fs_path_fs_path(&p1, &p2));
+    RE(atf_fs_path_init_fmt(&p2, "foo"));
+    ATF_REQUIRE(atf_equal_fs_path_fs_path(&p1, &p2));
     atf_fs_path_fini(&p2);
 
-    CE(atf_fs_path_init_fmt(&p2, "bar"));
-    ATF_CHECK(!atf_equal_fs_path_fs_path(&p1, &p2));
+    RE(atf_fs_path_init_fmt(&p2, "bar"));
+    ATF_REQUIRE(!atf_equal_fs_path_fs_path(&p1, &p2));
     atf_fs_path_fini(&p2);
 
     atf_fs_path_fini(&p1);
@@ -438,15 +438,15 @@ ATF_TC_BODY(stat_type, tc)
     create_dir("dir", 0755);
     create_file("reg", 0644);
 
-    CE(atf_fs_path_init_fmt(&p, "dir"));
-    CE(atf_fs_stat_init(&st, &p));
-    ATF_CHECK_EQUAL(atf_fs_stat_get_type(&st), atf_fs_stat_dir_type);
+    RE(atf_fs_path_init_fmt(&p, "dir"));
+    RE(atf_fs_stat_init(&st, &p));
+    ATF_REQUIRE_EQ(atf_fs_stat_get_type(&st), atf_fs_stat_dir_type);
     atf_fs_stat_fini(&st);
     atf_fs_path_fini(&p);
 
-    CE(atf_fs_path_init_fmt(&p, "reg"));
-    CE(atf_fs_stat_init(&st, &p));
-    ATF_CHECK_EQUAL(atf_fs_stat_get_type(&st), atf_fs_stat_reg_type);
+    RE(atf_fs_path_init_fmt(&p, "reg"));
+    RE(atf_fs_stat_init(&st, &p));
+    ATF_REQUIRE_EQ(atf_fs_stat_get_type(&st), atf_fs_stat_reg_type);
     atf_fs_stat_fini(&st);
     atf_fs_path_fini(&p);
 }
@@ -463,20 +463,20 @@ ATF_TC_BODY(stat_perms, tc)
 
     create_file("reg", 0);
 
-    CE(atf_fs_path_init_fmt(&p, "reg"));
+    RE(atf_fs_path_init_fmt(&p, "reg"));
 
 #define perms(ur, uw, ux, gr, gw, gx, othr, othw, othx) \
     { \
-        CE(atf_fs_stat_init(&st, &p)); \
-        ATF_CHECK(atf_fs_stat_is_owner_readable(&st) == ur); \
-        ATF_CHECK(atf_fs_stat_is_owner_writable(&st) == uw); \
-        ATF_CHECK(atf_fs_stat_is_owner_executable(&st) == ux); \
-        ATF_CHECK(atf_fs_stat_is_group_readable(&st) == gr); \
-        ATF_CHECK(atf_fs_stat_is_group_writable(&st) == gw); \
-        ATF_CHECK(atf_fs_stat_is_group_executable(&st) == gx); \
-        ATF_CHECK(atf_fs_stat_is_other_readable(&st) == othr); \
-        ATF_CHECK(atf_fs_stat_is_other_writable(&st) == othw); \
-        ATF_CHECK(atf_fs_stat_is_other_executable(&st) == othx); \
+        RE(atf_fs_stat_init(&st, &p)); \
+        ATF_REQUIRE(atf_fs_stat_is_owner_readable(&st) == ur); \
+        ATF_REQUIRE(atf_fs_stat_is_owner_writable(&st) == uw); \
+        ATF_REQUIRE(atf_fs_stat_is_owner_executable(&st) == ux); \
+        ATF_REQUIRE(atf_fs_stat_is_group_readable(&st) == gr); \
+        ATF_REQUIRE(atf_fs_stat_is_group_writable(&st) == gw); \
+        ATF_REQUIRE(atf_fs_stat_is_group_executable(&st) == gx); \
+        ATF_REQUIRE(atf_fs_stat_is_other_readable(&st) == othr); \
+        ATF_REQUIRE(atf_fs_stat_is_other_writable(&st) == othw); \
+        ATF_REQUIRE(atf_fs_stat_is_other_executable(&st) == othx); \
         atf_fs_stat_fini(&st); \
     }
 
@@ -543,9 +543,9 @@ ATF_TC_BODY(cleanup, tc)
     create_file("root/dir/2", 0644);
     create_file("root/reg", 0644);
 
-    CE(atf_fs_path_init_fmt(&root, "root"));
-    CE(atf_fs_cleanup(&root));
-    ATF_CHECK(not_exists(&root));
+    RE(atf_fs_path_init_fmt(&root, "root"));
+    RE(atf_fs_cleanup(&root));
+    ATF_REQUIRE(not_exists(&root));
     atf_fs_path_fini(&root);
 
     /* TODO: Cleanup with mount points, just as in tools/t_atf_cleanup. */
@@ -562,34 +562,34 @@ ATF_TC_BODY(exists, tc)
     atf_fs_path_t pdir, pfile;
     bool b;
 
-    CE(atf_fs_path_init_fmt(&pdir, "dir"));
-    CE(atf_fs_path_init_fmt(&pfile, "dir/file"));
+    RE(atf_fs_path_init_fmt(&pdir, "dir"));
+    RE(atf_fs_path_init_fmt(&pfile, "dir/file"));
 
     create_dir(atf_fs_path_cstring(&pdir), 0755);
     create_file(atf_fs_path_cstring(&pfile), 0644);
 
     printf("Checking existence of a directory\n");
-    CE(atf_fs_exists(&pdir, &b));
-    ATF_CHECK(b);
+    RE(atf_fs_exists(&pdir, &b));
+    ATF_REQUIRE(b);
 
     printf("Checking existence of a file\n");
-    CE(atf_fs_exists(&pfile, &b));
-    ATF_CHECK(b);
+    RE(atf_fs_exists(&pfile, &b));
+    ATF_REQUIRE(b);
 
     if (!atf_user_is_root()) {
         printf("Checking existence of a file inside a directory without "
                "permissions\n");
-        ATF_CHECK(chmod(atf_fs_path_cstring(&pdir), 0000) != -1);
+        ATF_REQUIRE(chmod(atf_fs_path_cstring(&pdir), 0000) != -1);
         err = atf_fs_exists(&pfile, &b);
-        ATF_CHECK(atf_is_error(err));
-        ATF_CHECK(atf_error_is(err, "libc"));
-        ATF_CHECK(chmod(atf_fs_path_cstring(&pdir), 0755) != -1);
+        ATF_REQUIRE(atf_is_error(err));
+        ATF_REQUIRE(atf_error_is(err, "libc"));
+        ATF_REQUIRE(chmod(atf_fs_path_cstring(&pdir), 0755) != -1);
     }
 
     printf("Checking existence of a non-existent file\n");
-    ATF_CHECK(unlink(atf_fs_path_cstring(&pfile)) != -1);
-    CE(atf_fs_exists(&pfile, &b));
-    ATF_CHECK(!b);
+    ATF_REQUIRE(unlink(atf_fs_path_cstring(&pfile)) != -1);
+    RE(atf_fs_exists(&pfile, &b));
+    ATF_REQUIRE(!b);
 }
 
 ATF_TC(eaccess);
@@ -648,32 +648,32 @@ ATF_TC_BODY(eaccess, tc)
     atf_fs_path_t p;
     atf_error_t err;
 
-    CE(atf_fs_path_init_fmt(&p, "the-file"));
+    RE(atf_fs_path_init_fmt(&p, "the-file"));
 
     printf("Non-existent file checks\n");
     for (m = &modes[0]; *m != 0; m++) {
         err = atf_fs_eaccess(&p, *m);
-        ATF_CHECK(atf_is_error(err));
-        ATF_CHECK(atf_error_is(err, "libc"));
-        ATF_CHECK_EQUAL(atf_libc_error_code(err), ENOENT);
+        ATF_REQUIRE(atf_is_error(err));
+        ATF_REQUIRE(atf_error_is(err, "libc"));
+        ATF_REQUIRE_EQ(atf_libc_error_code(err), ENOENT);
         atf_error_free(err);
     }
 
     create_file(atf_fs_path_cstring(&p), 0000);
-    ATF_CHECK(chown(atf_fs_path_cstring(&p), geteuid(), getegid()) != -1);
+    ATF_REQUIRE(chown(atf_fs_path_cstring(&p), geteuid(), getegid()) != -1);
 
     for (t = &tests[0]; t->amode != 0; t++) {
         const int experr = atf_user_is_root() ? t->rerror : t->uerror;
 
         printf("\n");
-        printf("File mode     : %04o\n", t->fmode);
+        printf("File mode     : %04o\n", (unsigned int)t->fmode);
         printf("Access mode   : 0x%02x\n", t->amode);
 
-        ATF_CHECK(chmod(atf_fs_path_cstring(&p), t->fmode) != -1);
+        ATF_REQUIRE(chmod(atf_fs_path_cstring(&p), t->fmode) != -1);
 
         /* First, existence check. */
         err = atf_fs_eaccess(&p, atf_fs_access_f);
-        ATF_CHECK(!atf_is_error(err));
+        ATF_REQUIRE(!atf_is_error(err));
 
         /* Now do the specific test case. */
         printf("Expected error: %d\n", experr);
@@ -686,11 +686,11 @@ ATF_TC_BODY(eaccess, tc)
         } else
                 printf("Error         : None\n");
         if (experr == 0) {
-            ATF_CHECK(!atf_is_error(err));
+            ATF_REQUIRE(!atf_is_error(err));
         } else {
-            ATF_CHECK(atf_is_error(err));
-            ATF_CHECK(atf_error_is(err, "libc"));
-            ATF_CHECK_EQUAL(atf_libc_error_code(err), experr);
+            ATF_REQUIRE(atf_is_error(err));
+            ATF_REQUIRE(atf_error_is(err, "libc"));
+            ATF_REQUIRE_EQ(atf_libc_error_code(err), experr);
             atf_error_free(err);
         }
     }
@@ -709,40 +709,125 @@ ATF_TC_BODY(getcwd, tc)
 
     create_dir ("root", 0755);
 
-    CE(atf_fs_getcwd(&cwd1));
-    ATF_CHECK(chdir("root") != -1);
-    CE(atf_fs_getcwd(&cwd2));
+    RE(atf_fs_getcwd(&cwd1));
+    ATF_REQUIRE(chdir("root") != -1);
+    RE(atf_fs_getcwd(&cwd2));
 
-    CE(atf_fs_path_append_fmt(&cwd1, "root"));
+    RE(atf_fs_path_append_fmt(&cwd1, "root"));
 
-    ATF_CHECK(atf_equal_fs_path_fs_path(&cwd1, &cwd2));
+    ATF_REQUIRE(atf_equal_fs_path_fs_path(&cwd1, &cwd2));
 
     atf_fs_path_fini(&cwd2);
     atf_fs_path_fini(&cwd1);
 }
 
-ATF_TC(mkdtemp);
-ATF_TC_HEAD(mkdtemp, tc)
+ATF_TC(mkdtemp_ok);
+ATF_TC_HEAD(mkdtemp_ok, tc)
 {
-    atf_tc_set_md_var(tc, "descr", "Tests the atf_fs_mkdtemp function");
+    atf_tc_set_md_var(tc, "descr", "Tests the atf_fs_mkdtemp function, "
+                      "successful execution");
 }
-ATF_TC_BODY(mkdtemp, tc)
+ATF_TC_BODY(mkdtemp_ok, tc)
 {
     atf_fs_path_t p1, p2;
     atf_fs_stat_t s1, s2;
 
-    CE(atf_fs_path_init_fmt(&p1, "testdir.XXXXXX"));
-    CE(atf_fs_path_init_fmt(&p2, "testdir.XXXXXX"));
-    CE(atf_fs_mkdtemp(&p1));
-    CE(atf_fs_mkdtemp(&p2));
-    ATF_CHECK(!atf_equal_fs_path_fs_path(&p1, &p2));
-    ATF_CHECK(exists(&p1));
-    ATF_CHECK(exists(&p2));
+    RE(atf_fs_path_init_fmt(&p1, "testdir.XXXXXX"));
+    RE(atf_fs_path_init_fmt(&p2, "testdir.XXXXXX"));
+    RE(atf_fs_mkdtemp(&p1));
+    RE(atf_fs_mkdtemp(&p2));
+    ATF_REQUIRE(!atf_equal_fs_path_fs_path(&p1, &p2));
+    ATF_REQUIRE(exists(&p1));
+    ATF_REQUIRE(exists(&p2));
 
-    CE(atf_fs_stat_init(&s1, &p1));
+    RE(atf_fs_stat_init(&s1, &p1));
+    ATF_REQUIRE_EQ(atf_fs_stat_get_type(&s1), atf_fs_stat_dir_type);
+    ATF_REQUIRE( atf_fs_stat_is_owner_readable(&s1));
+    ATF_REQUIRE( atf_fs_stat_is_owner_writable(&s1));
+    ATF_REQUIRE( atf_fs_stat_is_owner_executable(&s1));
+    ATF_REQUIRE(!atf_fs_stat_is_group_readable(&s1));
+    ATF_REQUIRE(!atf_fs_stat_is_group_writable(&s1));
+    ATF_REQUIRE(!atf_fs_stat_is_group_executable(&s1));
+    ATF_REQUIRE(!atf_fs_stat_is_other_readable(&s1));
+    ATF_REQUIRE(!atf_fs_stat_is_other_writable(&s1));
+    ATF_REQUIRE(!atf_fs_stat_is_other_executable(&s1));
+
+    RE(atf_fs_stat_init(&s2, &p2));
+    ATF_REQUIRE_EQ(atf_fs_stat_get_type(&s2), atf_fs_stat_dir_type);
+    ATF_REQUIRE( atf_fs_stat_is_owner_readable(&s2));
+    ATF_REQUIRE( atf_fs_stat_is_owner_writable(&s2));
+    ATF_REQUIRE( atf_fs_stat_is_owner_executable(&s2));
+    ATF_REQUIRE(!atf_fs_stat_is_group_readable(&s2));
+    ATF_REQUIRE(!atf_fs_stat_is_group_writable(&s2));
+    ATF_REQUIRE(!atf_fs_stat_is_group_executable(&s2));
+    ATF_REQUIRE(!atf_fs_stat_is_other_readable(&s2));
+    ATF_REQUIRE(!atf_fs_stat_is_other_writable(&s2));
+    ATF_REQUIRE(!atf_fs_stat_is_other_executable(&s2));
+
+    atf_fs_stat_fini(&s2);
+    atf_fs_stat_fini(&s1);
+    atf_fs_path_fini(&p2);
+    atf_fs_path_fini(&p1);
+}
+
+ATF_TC(mkdtemp_err);
+ATF_TC_HEAD(mkdtemp_err, tc)
+{
+    atf_tc_set_md_var(tc, "descr", "Tests the atf_fs_mkdtemp function, "
+                      "error conditions");
+    atf_tc_set_md_var(tc, "require.user", "unprivileged");
+}
+ATF_TC_BODY(mkdtemp_err, tc)
+{
+    atf_error_t err;
+    atf_fs_path_t p;
+
+    ATF_REQUIRE(mkdir("dir", 0555) != -1);
+
+    RE(atf_fs_path_init_fmt(&p, "dir/testdir.XXXXXX"));
+
+    err = atf_fs_mkdtemp(&p);
+    ATF_REQUIRE(atf_is_error(err));
+    ATF_REQUIRE(atf_error_is(err, "libc"));
+    ATF_CHECK_EQ(atf_libc_error_code(err), EACCES);
+
+    ATF_CHECK(!exists(&p));
+    ATF_CHECK(strcmp(atf_fs_path_cstring(&p), "dir/testdir.XXXXXX") == 0);
+}
+
+ATF_TC(mkstemp_ok);
+ATF_TC_HEAD(mkstemp_ok, tc)
+{
+    atf_tc_set_md_var(tc, "descr", "Tests the atf_fs_mkstemp function, "
+                      "successful execution");
+}
+ATF_TC_BODY(mkstemp_ok, tc)
+{
+    int fd1, fd2;
+    atf_fs_path_t p1, p2;
+    atf_fs_stat_t s1, s2;
+
+    RE(atf_fs_path_init_fmt(&p1, "testfile.XXXXXX"));
+    RE(atf_fs_path_init_fmt(&p2, "testfile.XXXXXX"));
+    fd1 = fd2 = -1;
+    RE(atf_fs_mkstemp(&p1, &fd1));
+    RE(atf_fs_mkstemp(&p2, &fd2));
+    ATF_REQUIRE(!atf_equal_fs_path_fs_path(&p1, &p2));
+    ATF_REQUIRE(exists(&p1));
+    ATF_REQUIRE(exists(&p2));
+
+    ATF_CHECK(fd1 != -1);
+    ATF_CHECK(fd2 != -1);
+    ATF_CHECK(write(fd1, "foo", 3) == 3);
+    ATF_CHECK(write(fd2, "bar", 3) == 3);
+    close(fd1);
+    close(fd2);
+
+    RE(atf_fs_stat_init(&s1, &p1));
+    ATF_CHECK_EQ(atf_fs_stat_get_type(&s1), atf_fs_stat_reg_type);
     ATF_CHECK( atf_fs_stat_is_owner_readable(&s1));
     ATF_CHECK( atf_fs_stat_is_owner_writable(&s1));
-    ATF_CHECK( atf_fs_stat_is_owner_executable(&s1));
+    ATF_CHECK(!atf_fs_stat_is_owner_executable(&s1));
     ATF_CHECK(!atf_fs_stat_is_group_readable(&s1));
     ATF_CHECK(!atf_fs_stat_is_group_writable(&s1));
     ATF_CHECK(!atf_fs_stat_is_group_executable(&s1));
@@ -750,10 +835,11 @@ ATF_TC_BODY(mkdtemp, tc)
     ATF_CHECK(!atf_fs_stat_is_other_writable(&s1));
     ATF_CHECK(!atf_fs_stat_is_other_executable(&s1));
 
-    CE(atf_fs_stat_init(&s2, &p2));
+    RE(atf_fs_stat_init(&s2, &p2));
+    ATF_CHECK_EQ(atf_fs_stat_get_type(&s2), atf_fs_stat_reg_type);
     ATF_CHECK( atf_fs_stat_is_owner_readable(&s2));
     ATF_CHECK( atf_fs_stat_is_owner_writable(&s2));
-    ATF_CHECK( atf_fs_stat_is_owner_executable(&s2));
+    ATF_CHECK(!atf_fs_stat_is_owner_executable(&s2));
     ATF_CHECK(!atf_fs_stat_is_group_readable(&s2));
     ATF_CHECK(!atf_fs_stat_is_group_writable(&s2));
     ATF_CHECK(!atf_fs_stat_is_group_executable(&s2));
@@ -767,6 +853,34 @@ ATF_TC_BODY(mkdtemp, tc)
     atf_fs_path_fini(&p1);
 }
 
+ATF_TC(mkstemp_err);
+ATF_TC_HEAD(mkstemp_err, tc)
+{
+    atf_tc_set_md_var(tc, "descr", "Tests the atf_fs_mkstemp function, "
+                      "error conditions");
+    atf_tc_set_md_var(tc, "require.user", "unprivileged");
+}
+ATF_TC_BODY(mkstemp_err, tc)
+{
+    int fd;
+    atf_error_t err;
+    atf_fs_path_t p;
+
+    ATF_REQUIRE(mkdir("dir", 0555) != -1);
+
+    RE(atf_fs_path_init_fmt(&p, "dir/testfile.XXXXXX"));
+    fd = 1234;
+
+    err = atf_fs_mkstemp(&p, &fd);
+    ATF_REQUIRE(atf_is_error(err));
+    ATF_REQUIRE(atf_error_is(err, "libc"));
+    ATF_CHECK_EQ(atf_libc_error_code(err), EACCES);
+
+    ATF_CHECK(!exists(&p));
+    ATF_CHECK(strcmp(atf_fs_path_cstring(&p), "dir/testfile.XXXXXX") == 0);
+    ATF_CHECK_EQ(fd, 1234);
+}
+
 /* ---------------------------------------------------------------------
  * Main.
  * --------------------------------------------------------------------- */
@@ -793,7 +907,10 @@ ATF_TP_ADD_TCS(tp)
     ATF_TP_ADD_TC(tp, eaccess);
     ATF_TP_ADD_TC(tp, exists);
     ATF_TP_ADD_TC(tp, getcwd);
-    ATF_TP_ADD_TC(tp, mkdtemp);
+    ATF_TP_ADD_TC(tp, mkdtemp_ok);
+    ATF_TP_ADD_TC(tp, mkdtemp_err);
+    ATF_TP_ADD_TC(tp, mkstemp_ok);
+    ATF_TP_ADD_TC(tp, mkstemp_err);
 
     return atf_no_error();
 }
