@@ -1,7 +1,7 @@
 /*
  * Automated Testing Framework (atf)
  *
- * Copyright (c) 2007, 2008 The NetBSD Foundation, Inc.
+ * Copyright (c) 2007, 2008, 2009 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,15 +41,16 @@
 #include <atf-c.h>
 
 #include "atf-c/env.h"
+#include "atf-c/error.h"
 #include "atf-c/fs.h"
 #include "atf-c/text.h"
 
+#include "../atf-c/h_macros.h"
+
 /* ---------------------------------------------------------------------
  * Auxiliary functions.
  * --------------------------------------------------------------------- */
 
-#define CE(stm) ATF_CHECK(!atf_is_error(stm))
-
 static
 void
 write_cwd(const atf_tc_t *tc, const char *confvar)
@@ -64,7 +65,7 @@ write_cwd(const atf_tc_t *tc, const char *confvar)
     if (f == NULL)
         atf_tc_fail("Could not open %s for writing", p);
 
-    CE(atf_fs_getcwd(&cwd));
+    RE(atf_fs_getcwd(&cwd));
     fprintf(f, "%s\n", atf_fs_path_cstring(&cwd));
     atf_fs_path_fini(&cwd);
 
@@ -116,7 +117,7 @@ ATF_TC_CLEANUP(cleanup_pass, tc)
 {
     bool cleanup;
 
-    CE(atf_text_to_bool(atf_tc_get_config_var(tc, "cleanup"), &cleanup));
+    RE(atf_text_to_bool(atf_tc_get_config_var(tc, "cleanup"), &cleanup));
 
     if (cleanup)
         safe_remove(atf_tc_get_config_var(tc, "tmpfile"));
@@ -137,7 +138,7 @@ ATF_TC_CLEANUP(cleanup_fail, tc)
 {
     bool cleanup;
 
-    CE(atf_text_to_bool(atf_tc_get_config_var(tc, "cleanup"), &cleanup));
+    RE(atf_text_to_bool(atf_tc_get_config_var(tc, "cleanup"), &cleanup));
 
     if (cleanup)
         safe_remove(atf_tc_get_config_var(tc, "tmpfile"));
@@ -158,7 +159,7 @@ ATF_TC_CLEANUP(cleanup_skip, tc)
 {
     bool cleanup;
 
-    CE(atf_text_to_bool(atf_tc_get_config_var(tc, "cleanup"), &cleanup));
+    RE(atf_text_to_bool(atf_tc_get_config_var(tc, "cleanup"), &cleanup));
 
     if (cleanup)
         safe_remove(atf_tc_get_config_var(tc, "tmpfile"));
@@ -208,7 +209,7 @@ ATF_TC_BODY(cleanup_sigterm, tc)
     touch(atf_tc_get_config_var(tc, "tmpfile"));
     kill(getpid(), SIGTERM);
 
-    CE(atf_text_format(&nofile, "%s.no",
+    RE(atf_text_format(&nofile, "%s.no",
                        atf_tc_get_config_var(tc, "tmpfile")));
     touch(nofile);
     free(nofile);
@@ -246,7 +247,7 @@ ATF_TC_HEAD(config_unset, tc)
 }
 ATF_TC_BODY(config_unset, tc)
 {
-    ATF_CHECK(!atf_tc_has_config_var(tc, "test"));
+    ATF_REQUIRE(!atf_tc_has_config_var(tc, "test"));
 }
 
 ATF_TC(config_empty);
@@ -257,8 +258,8 @@ ATF_TC_HEAD(config_empty, tc)
 }
 ATF_TC_BODY(config_empty, tc)
 {
-    ATF_CHECK(atf_tc_has_config_var(tc, "test"));
-    ATF_CHECK(strlen(atf_tc_get_config_var(tc, "test")) == 0);
+    ATF_REQUIRE(atf_tc_has_config_var(tc, "test"));
+    ATF_REQUIRE(strlen(atf_tc_get_config_var(tc, "test")) == 0);
 }
 
 ATF_TC(config_value);
@@ -269,8 +270,8 @@ ATF_TC_HEAD(config_value, tc)
 }
 ATF_TC_BODY(config_value, tc)
 {
-    ATF_CHECK(atf_tc_has_config_var(tc, "test"));
-    ATF_CHECK(strcmp(atf_tc_get_config_var(tc, "test"), "foo") == 0);
+    ATF_REQUIRE(atf_tc_has_config_var(tc, "test"));
+    ATF_REQUIRE(strcmp(atf_tc_get_config_var(tc, "test"), "foo") == 0);
 }
 
 ATF_TC(config_multi_value);
@@ -281,8 +282,8 @@ ATF_TC_HEAD(config_multi_value, tc)
 }
 ATF_TC_BODY(config_multi_value, tc)
 {
-    ATF_CHECK(atf_tc_has_config_var(tc, "test"));
-    ATF_CHECK(strcmp(atf_tc_get_config_var(tc, "test"), "foo bar") == 0);
+    ATF_REQUIRE(atf_tc_has_config_var(tc, "test"));
+    ATF_REQUIRE(strcmp(atf_tc_get_config_var(tc, "test"), "foo bar") == 0);
 }
 
 /* ---------------------------------------------------------------------
@@ -300,17 +301,17 @@ ATF_TC_BODY(env_home, tc)
     atf_fs_path_t cwd, home;
     atf_fs_stat_t stcwd, sthome;
 
-    ATF_CHECK(atf_env_has("HOME"));
+    ATF_REQUIRE(atf_env_has("HOME"));
 
-    CE(atf_fs_getcwd(&cwd));
-    CE(atf_fs_path_init_fmt(&home, "%s", atf_env_get("HOME")));
+    RE(atf_fs_getcwd(&cwd));
+    RE(atf_fs_path_init_fmt(&home, "%s", atf_env_get("HOME")));
 
-    CE(atf_fs_stat_init(&stcwd, &cwd));
-    CE(atf_fs_stat_init(&sthome, &home));
+    RE(atf_fs_stat_init(&stcwd, &cwd));
+    RE(atf_fs_stat_init(&sthome, &home));
 
-    ATF_CHECK_EQUAL(atf_fs_stat_get_device(&stcwd),
+    ATF_REQUIRE_EQ(atf_fs_stat_get_device(&stcwd),
                     atf_fs_stat_get_device(&sthome));
-    ATF_CHECK_EQUAL(atf_fs_stat_get_inode(&stcwd),
+    ATF_REQUIRE_EQ(atf_fs_stat_get_inode(&stcwd),
                     atf_fs_stat_get_inode(&sthome));
 }
 
@@ -323,8 +324,8 @@ ATF_TC_HEAD(env_list, tc)
 ATF_TC_BODY(env_list, tc)
 {
     int exitcode = system("env");
-    ATF_CHECK(WIFEXITED(exitcode));
-    ATF_CHECK(WEXITSTATUS(exitcode) == EXIT_SUCCESS);
+    ATF_REQUIRE(WIFEXITED(exitcode));
+    ATF_REQUIRE(WEXITSTATUS(exitcode) == EXIT_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------
@@ -341,7 +342,7 @@ ATF_TC_BODY(fork_mangle_fds, tc)
 {
     long resfd;
 
-    CE(atf_text_to_long(atf_tc_get_config_var(tc, "resfd"), &resfd));
+    RE(atf_text_to_long(atf_tc_get_config_var(tc, "resfd"), &resfd));
 
     if (close(STDIN_FILENO) == -1)
         atf_tc_fail("Failed to close stdin");
@@ -375,7 +376,7 @@ ATF_TC_BODY(fork_stop, tc)
     f = fopen(pfstr, "w");
     if (f == NULL)
         atf_tc_fail("Failed to create pidfile %s", pfstr);
-    fprintf(f, "%d", getpid());
+    fprintf(f, "%d", (int)getpid());
     fclose(f);
     printf("Wrote pid file\n");
 
@@ -393,7 +394,7 @@ ATF_TC_HEAD(fork_umask, tc)
 }
 ATF_TC_BODY(fork_umask, tc)
 {
-    printf("umask: %04o\n", umask(0));
+    printf("umask: %04o\n", (unsigned int)umask(0));
 }
 
 /* ---------------------------------------------------------------------
@@ -408,7 +409,7 @@ ATF_TC_HEAD(ident_1, tc)
 }
 ATF_TC_BODY(ident_1, tc)
 {
-    ATF_CHECK(strcmp(atf_tc_get_md_var(tc, "ident"), "ident_1") == 0);
+    ATF_REQUIRE(strcmp(atf_tc_get_md_var(tc, "ident"), "ident_1") == 0);
 }
 
 ATF_TC(ident_2);
@@ -419,7 +420,7 @@ ATF_TC_HEAD(ident_2, tc)
 }
 ATF_TC_BODY(ident_2, tc)
 {
-    ATF_CHECK(strcmp(atf_tc_get_md_var(tc, "ident"), "ident_2") == 0);
+    ATF_REQUIRE(strcmp(atf_tc_get_md_var(tc, "ident"), "ident_2") == 0);
 }
 
 ATF_TC(require_arch);
@@ -530,7 +531,7 @@ ATF_TC_BODY(timeout, tc)
 {
     long s;
 
-    CE(atf_text_to_long(atf_tc_get_config_var(tc, "sleep"), &s));
+    RE(atf_text_to_long(atf_tc_get_config_var(tc, "sleep"), &s));
     sleep(s);
 }
 
@@ -546,7 +547,7 @@ ATF_TC_BODY(timeout2, tc)
 {
     long s;
 
-    CE(atf_text_to_long(atf_tc_get_config_var(tc, "sleep2"), &s));
+    RE(atf_text_to_long(atf_tc_get_config_var(tc, "sleep2"), &s));
     sleep(s);
 }
 
@@ -565,9 +566,9 @@ ATF_TC_BODY(srcdir_exists, tc)
     atf_fs_path_t p;
     bool b;
 
-    CE(atf_fs_path_init_fmt(&p, "%s/datafile",
+    RE(atf_fs_path_init_fmt(&p, "%s/datafile",
                             atf_tc_get_config_var(tc, "srcdir")));
-    CE(atf_fs_exists(&p, &b));
+    RE(atf_fs_exists(&p, &b));
     if (!b)
         atf_tc_fail("Cannot find datafile");
     atf_fs_path_fini(&p);
