@@ -27,9 +27,31 @@
 # IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 
-cleanup()
+cleanup="$(atf-config -t atf_libexecdir)/atf-cleanup"
+
+create_mount_helper()
 {
-    $(atf-config -t atf_libexecdir)/atf-cleanup "${@}"
+    echo "#! $(atf-config -t atf_shell)" >${1}
+
+    platform=$(uname)
+    case ${platform} in
+    Linux|NetBSD)
+        echo 'mount -t tmpfs tmpfs ${1}' >>${1}
+        ;;
+    FreeBSD)
+        echo 'mdmfs -s 16m md ${1}' >>${1}
+        ;;
+    SunOS)
+        echo 'mount -F tmpfs tmpfs $(pwd)/${1}' >>${1}
+        ;;
+    *)
+        echo "create_mount_helper called for an unsupported platform."
+        echo "This should not have happened."
+        exit 1
+        ;;
+    esac
+
+    chmod +x ${1}
 }
 
 atf_test_case file
@@ -39,9 +61,9 @@ file_head()
 }
 file_body()
 {
-    atf_check 'touch foo' 0 null null
-    atf_check 'cleanup foo' 0 null null
-    atf_check 'test -e foo' 1 null null
+    atf_check -s eq:0 -o empty -e empty touch foo
+    atf_check -s eq:0 -o empty -e empty "${cleanup}" foo
+    atf_check -s eq:1 -o empty -e empty test -e foo
 }
 
 atf_test_case dir_empty
@@ -51,9 +73,9 @@ dir_empty_head()
 }
 dir_empty_body()
 {
-    atf_check 'mkdir foo' 0 null null
-    atf_check 'cleanup foo' 0 null null
-    atf_check 'test -e foo' 1 null null
+    atf_check -s eq:0 -o empty -e empty mkdir foo
+    atf_check -s eq:0 -o empty -e empty "${cleanup}" foo
+    atf_check -s eq:1 -o empty -e empty test -e foo
 }
 
 atf_test_case dir_full
@@ -63,31 +85,12 @@ dir_full_head()
 }
 dir_full_body()
 {
-    atf_check 'mkdir foo' 0 null null
-    atf_check 'mkdir foo/bar' 0 null null
-    atf_check 'touch foo/bar/baz' 0 null null
-    atf_check 'touch foo/baz' 0 null null
-    atf_check 'cleanup foo' 0 null null
-    atf_check 'test -e foo' 1 null null
-}
-
-mount_tmpfs_portable()
-{
-    platform=$(uname)
-    case ${platform} in
-    Linux|NetBSD)
-        mount -t tmpfs tmpfs ${1}
-        ;;
-    FreeBSD)
-        mdmfs -s 16m md ${1}
-        ;;
-    SunOS)
-        mount -F tmpfs tmpfs $(pwd)/${1}
-        ;;
-    *)
-        atf_fail "Internal error in test program"
-        ;;
-    esac
+    atf_check -s eq:0 -o empty -e empty mkdir foo
+    atf_check -s eq:0 -o empty -e empty mkdir foo/bar
+    atf_check -s eq:0 -o empty -e empty touch foo/bar/baz
+    atf_check -s eq:0 -o empty -e empty touch foo/baz
+    atf_check -s eq:0 -o empty -e empty "${cleanup}" foo
+    atf_check -s eq:1 -o empty -e empty test -e foo
 }
 
 atf_test_case mount
@@ -99,18 +102,21 @@ mount_head()
 }
 mount_body()
 {
+    create_mount_helper h_mount.sh
+
     platform=$(uname)
     case ${platform} in
     Linux|FreeBSD|NetBSD|SunOS)
         mkdir foo
         mkdir foo/bar
         mkdir foo/bar/mnt
-        atf_check 'mount_tmpfs_portable foo/bar/mnt' 0 null null
+        atf_check -s eq:0 -o empty -e empty -x "./h_mount.sh foo/bar/mnt"
         mkdir foo/baz
-        atf_check 'mount_tmpfs_portable foo/baz' 0 null null
+        atf_check -s eq:0 -o empty -e empty -x "./h_mount.sh foo/baz"
         mkdir foo/baz/foo
         mkdir foo/baz/foo/bar
-        atf_check 'mount_tmpfs_portable foo/baz/foo/bar' 0 null null
+        atf_check -s eq:0 -o empty -e empty -x \
+                  "./h_mount.sh foo/baz/foo/bar"
         ;;
     *)
         # XXX Possibly specify in meta-data too.
@@ -118,10 +124,10 @@ mount_body()
         ;;
     esac
 
-    atf_check "cleanup foo" 0 null null
+    atf_check -s eq:0 -o empty -e empty "${cleanup}" foo
     mount | grep $(pwd)/foo && \
         atf_fail "Some file systems remain mounted"
-    atf_check "test -d foo" 1 null null
+    atf_check -s eq:1 -o empty -e empty test -d foo
 }
 
 atf_test_case symlink
@@ -134,14 +140,16 @@ symlink_head()
 }
 symlink_body()
 {
+    create_mount_helper h_mount.sh
+
     platform=$(uname)
     case ${platform} in
     Linux|FreeBSD|NetBSD|SunOS)
-        atf_check 'mkdir foo' 0 null null
-        atf_check 'mkdir foo/bar' 0 null null
-        atf_check 'mount_tmpfs_portable foo/bar' 0 null null
-        atf_check 'touch a' 0 null null
-        atf_check 'ln -s $(pwd)/a foo/bar' 0 null null
+        atf_check -s eq:0 -o empty -e empty mkdir foo
+        atf_check -s eq:0 -o empty -e empty mkdir foo/bar
+        atf_check -s eq:0 -o empty -e empty -x "./h_mount.sh foo/bar"
+        atf_check -s eq:0 -o empty -e empty touch a
+        atf_check -s eq:0 -o empty -e empty ln -s "$(pwd)/a" foo/bar
         ;;
     *)
         # XXX Possibly specify in meta-data too.
@@ -149,10 +157,10 @@ symlink_body()
         ;;
     esac
 
-    atf_check "cleanup foo" 0 null null
+    atf_check -s eq:0 -o empty -e empty ${cleanup} foo
     mount | grep $(pwd)/foo && \
         atf_fail "Some file systems remain mounted"
-    atf_check "test -d foo" 1 null null
+    atf_check -s eq:1 -o empty -e empty test -d foo
 }
 
 atf_init_test_cases()
